from App.Common import package_home
from StringIO import StringIO
from Products.CMFCore.utils import getToolByName, minimalpath
#from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.TypesTool import FactoryTypeInformation as ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews, createDirectoryView
from Products.CMFOODocument import oodocument_globals as skins_globals, ARCHETYPES_MODE
from Products.CMFOODocument.oo_to_html import _addtypes as openoffice_mime_types
from Products.CMFOODocument import ftypes
import string

try:
    from Products.Archetypes.public import *
    from Products.Archetypes.Extensions.utils import installTypes, install_subskin
    from Products.Archetypes import listTypes
except ImportError:
    ARCHETYPES_MODE=0


PKG_NAME = "CMFOODocument"


def install(self):
    """ run this method to install CMFOODocument to your portal """
    out=StringIO()

    if not hasattr(self, "_isPortalRoot"):
        print >> out, "CMFOODocument must be installed into a CMF Site"
        return out.getvalue()

    print >> out, "Installing CMF OpenOffice.org Document"
    print >> out, "========================================\n"

    if ARCHETYPES_MODE: 
        installTypes(self, out, listTypes(PKG_NAME), PKG_NAME)
        install_subskin(self, out, skins_globals)

    # add CMFOODocument to types tool
    portal_types = getToolByName(self, 'portal_types')

    if not ARCHETYPES_MODE:
        for f in ftypes:
            if f['id'] not in portal_types.objectIds():
                cfm = apply(ContentFactoryMetadata, (), f)
                portal_types._setObject(f['id'], cfm)
                print >> out, 'Registered "%s" in the types tool\n' % (f['id'])
            else:
                print >> out, 'Object "%s" already existed in the types tool' % (f['id'])
                print >> out, 'NOTE: if this is a upgrade you should probably delete the factory first\n'

    ti = portal_types.getTypeInfo('I18NLayer')
    if ti:
        if ti.filter_content_types:
            # add OODocument and OOODocument to the I18NLayer
            for t in ['OODocument', 'OOODocument']:
                if t in ti.allowed_content_types:
                    print >> out, '%s already allowed to be added inside I18NLayers.\n' % t
                else:
                    allowed=ti.allowed_content_types
                    if type(allowed) == type((1,)):
                        l=ti.allowed_content_types+(t,)
                    elif type(allowed) == type([1,]):
                        l=ti.allowed_content_types+[t,]
                    ti.allowed_content_types=l
                    print >> out, 'Added %s to I18NLayers allowed Content Types.\n' % t
        
    # add skins
    skinstool = getToolByName(self, 'portal_skins')
    for skinpath in ('OODocument',):

        if skinpath not in skinstool.objectIds():
            # We need to add Filesystem Directory Views for any directories
            # in our skins/ directory.  These directories should already be
            # configured.
            fp = minimalpath("%s/skins/%s" % (package_home(skins_globals), skinpath))
            createDirectoryView(skinstool, fp, skinpath)
            print >> out, "Added %s directory view to portal_skins\n" % skinpath
        else:
            print >> out, "%s directory view already exists in portal_skins\n" % skinpath

        # Now we need to go through the skin configurations and insert
        # skinpath into the configurations.  Preferably, this
        # should be right after where 'custom' is placed.  Otherwise, we
        # append it to the end.
        skins = skinstool.getSkinSelections()
        for skin in skins:
            path = skinstool.getSkinPath(skin)
            path = map(string.strip, string.split(path,','))
            if skinpath not in path:
                try:
                    path.insert(path.index('custom')+1, skinpath)
                except ValueError:
                    path.append(skinpath)

                path = string.join(path, ', ')
                # addSkinSelection will replace existing skins as well.
                skinstool.addSkinSelection(skin, path)
                print >> out, "Added %s to %s skin\n" % (skinpath, skin)
            else:
                print >> out, "Skipping %s skin, %s is already set up\n" % (skin, skinpath)

    try:
        # register openoffice mimetypes to transform mimetypes registry
        from Products.PortalTransforms.MimeTypeItem import MimeTypeItem
        mimetypes_tool = self.mimetypes_registry
        for ext, name in openoffice_mime_types:
            if not len(mimetypes_tool.lookup(name)):
                mt=MimeTypeItem(name, (name,), (ext[1:],), binary=1)
                mimetypes_tool.register(mt)
                print >> out, "Registered mimetype %s\n" % name
            else:
                print >> out, "Already have mimetype %s registered\n" % name
    except:
        print >> out, "Unable to register openoffice mimetypes .. probably PortalTransforms Product is not installed (optional)\n"

    try:
        # register the oo_to_html transformer at the transform tool
        transform_tool = self.portal_transforms
        transform_tool.manage_addTransform('oo_to_html', 'Products.CMFOODocument.oo_to_html')
        print >> out, "Registered oo to html transformer to transformer tool\n"
    except:
        print >> out, "Unable to register the transfomer .. probably PortalTransforms Product is not installed (optional)\n"

    return out.getvalue()
    
