##############################################################################
#
# Copyright (c) 2001-2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""$Id: MimeTextField.py,v 1.4 2004/10/09 23:29:21 longsleep Exp $
"""

from Products.Archetypes.Field import TextField, ObjectField, Field
from Products.Archetypes.interfaces.base import IBaseUnit
from Products.Archetypes.BaseUnit import BaseUnit

from Acquisition import aq_base
from ZPublisher.HTTPRequest import FileUpload
from types import ListType, TupleType, ClassType, FileType, StringTypes
#from OFS.content_types import guess_content_type
from zope.contenttype import guess_content_type
from Products.Archetypes.interfaces.base import IBaseUnit
from Products.Archetypes.exceptions import TextFieldException

class MimeTextField(TextField):
    """ Field for objects that rely on exactly one type of transformation """
    __implements__ = TextField.__implements__

    _properties = Field._properties.copy()
    _properties.update({
        'type' : 'text',
        'default' : '',
        'default_content_type' : 'text/plain',
        'output_type'  : 'text/plain',
        'allowable_content_types' : ('text/plain',),
        'primary' : 0,
        })

    def defaultView(self):
        return self.output_type

    def get(self, instance, mimetype=None, raw=0, **kwargs):
        """ retrieve the transformed result """

        if raw: name=self.getName()
        else: name="%s-transformed" % self.getName()

        try:
            kwargs['field'] = self
            return self.storage.get(name, instance, **kwargs)
        except AttributeError:
            # happens if new Attrs are added and not yet stored in the instance
            if not kwargs.get('_initializing_', 0):
                self.set(instance, self.default,_initializing_=1,**kwargs)
            return ''

    def set(self, instance, value, **kwargs):
        """ transform to output_type """

        if not kwargs.has_key('mimetype'):
            kwargs['mimetype'] = None

        encoding = kwargs.get('encoding') or instance.portal_properties.site_properties.getProperty('default_charset')
        kwargs['encoding'] = encoding

        value, mimetype = self._process_input(value, default=self.default, **kwargs)

        if IBaseUnit.isImplementedBy(value):
            bu = value
        else:
            bu = BaseUnit(self.getName(), value, mimetype=mimetype, encoding=encoding, instance=instance)

        if not hasattr(bu, 'transform'):
            # probably old baseunit and these are not supported
            raise RuntimeError, "this data does not support transforming"
 
        # store baseunit
        ObjectField.set(self, instance, bu, **kwargs)

        mimetype =  self.output_type or 'text/plain'
        kwargs['mimetype'] = mimetype
        data = bu.transform(instance, mimetype)

        if not data: data=''

        # store transformed result
        # XXX: this isnt probably a good way to store additional data
        kwargs['field'] = self
        # Remove acquisition wrappers
        self.storage.set("%s-transformed" % self.getName(), instance, data, **kwargs)

    def _process_input(self, value, default=None, mimetype=None, encoding=None, **kwargs):
        # We also need to handle the case where there is a baseUnit
        # for this field containing a valid set of data that would
        # not be reuploaded in a subsequent edit, this is basically
        # migrated from the old BaseObject.set method
        if type(value) in StringTypes:
            if encoding is not None and type(value) == type(u''):
                value = value.encode(encoding)
            else:
                value = str(value)
            if mimetype is None:
                mimetype, enc = guess_content_type('', value, mimetype)
            if not value:
                return default, mimetype
            return value, mimetype
        else:
            if ((isinstance(value, FileUpload) and value.filename != '') or
                (isinstance(value, FileType) and value.name != '')):
                #OK, its a file, is it empty?
                f_name = ''
                if isinstance(value, FileUpload):
                    f_name = value.filename
                if isinstance(value, FileType):
                    f_name = value.name
                value = value.read()
                if mimetype is None:
                    mimetype, enc = guess_content_type(f_name, value, mimetype)
                size = len(value)
                if size == 0:
                    # This new file has no length, so we keep
                    # the orig
                    return default, mimetype
                return value, mimetype
            elif IBaseUnit.isImplementedBy(value):
                if mimetype is None:
                    mimetype, enc = guess_content_type('', str(value), mimetype)
                return value, getattr(aq_base(value), 'mimetype', mimetype)

        raise TextFieldException('Value is not File, String or BaseUnit on %s: %r' % (self.getName(), type(value)))







