##############################################################################
#
# Copyright (c) 2001-2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""$Id: OODocument.py,v 1.8 2004/10/09 22:46:51 longsleep Exp $
"""

# html rendered OpenOffice.org/StarOffice Documents 
# for CMF/Plone/Icoya written by Simon Eisenmann

import Globals, string, re, os, sys, time
from Globals import DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager

from Acquisition import aq_base, aq_parent
#from OFS.content_types import guess_content_type
from zope.contenttype import guess_content_type

import OFS.Image
from BTrees.OOBTree import OOBTree

#from Products.CMFCore.CMFCorePermissions import View
#from Products.CMFCore.CMFCorePermissions import ModifyPortalContent
from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import ModifyPortalContent
#from Products.CMFCore.utils import _verifyActionPermissions
from Products.CMFDefault.Document import Document
from Products.CMFDefault.File import File
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFDefault.utils import _dtmldir
from ComputedAttribute import ComputedAttribute
import oo_to_html
import utils


factory_type_information = { 'id'                   : 'OODocument'
                             , 'meta_type'          : 'OODocument'
                             , 'description'        : """\
                                                      OODocuments contain Documents written with\
                                                      OpenOffice.org / StarOffice. The content is rendered as html.
                                                      """
                             , 'icon'               : 'ooo_icon.png'
                             , 'product'            : 'CMFOODocument'
                             , 'factory'            : 'addOODocument'
                             , 'immediate_view'     : 'file_edit_form'
                             , 'actions'            :
                                ( { 'id'            : 'view' 
                                  , 'name'          : 'View'
                                  , 'action'        : 'oodocument_view'
                                  , 'permissions'   : (View,)
                                  }
                                , { 'id'	    : 'download'
                                  , 'name'	    : 'Download'
                                  , 'action'	    : 'file_view'
                                  , 'permissions'   : (ModifyPortalContent,)
                                  , 'visible'	    : 1
                                  }
                                , { 'id'            : 'edit'
                                  , 'name'          : 'Edit'
                                  , 'action'        : 'file_edit_form'
                                  , 'permissions'   : (ModifyPortalContent,)
                                  }
                                , { 'id'            : 'metadata'
                                  , 'name'          : 'Metadata'
                                  , 'action'        : 'metadata_edit_form'
                                  , 'permissions'   : (ModifyPortalContent,)
                                  }
                                , { 'id'            : 'external_edit'
                                  , 'name'          : 'External Edit'
                                  , 'action'        : 'external_edit'
                                  , 'permissions'   : (ModifyPortalContent,)
                                  , 'visible'       : 0
                                  }
                                )
                             }

def addOODocument( self
                 , id
                 , title=''
                 , file=''
                 , content_type=''
                 , precondition=''
                 , subject=() 
                 , description=''
                 , contributors=()
                 , effective_date=None
                 , expiration_date=None
                 , format=None
                 , language=''
                 , rights=''
                        ):
    """ Add a OODocument """

    # cookId sets the id and tile if they are not set
    id, title = OFS.Image.cookId(id, title, file)

    self=self.this()

    o = OODocument( id, title, '', content_type, precondition, subject
                   , description, contributors, effective_date, expiration_date
                   , format, language, rights
                   )
   
    # Add the OODocument instance to self
    self._setObject(id, o)

    # 'Upload' the file. This is done now rather than in the
    # contructor because the object is now in the ZODB and
    # can span ZODB objecs
    o=self._getOb(id)
    if not file:
        file=oo_to_html.loadEmptyOODocument()
        o.edit(file=file)
        o.content_type='application/vnd.sun.xml.writer'


class OODocument(File):
    """
    A Document which holds and renders
    OpenOffice.org / StarOffice files to html
    """

    meta_type = 'OODocument'
    effective_date = expiration_date = None
    _isDiscussable = 1
    text_format='html'
    css_styles=''
    _images=None

    security = ClassSecurityInfo()
    
    # define text attribute for compatibility with CMFDefault.Document view templates
    __str__ = File.__str__
    text = ComputedAttribute(__str__, 1)
    
    security.declareProtected(View, 'index_html')
    def index_html(self, REQUEST=None, RESPONSE={}):
        """
        Invokes the default view.
        """
        view = _getViewFor(self, 'view', 'document_view')
        if getattr(aq_base(view), 'isDocTemp', 0):
            return apply(view, (self, self.REQUEST))
        else:
            return view()

    __call__=view=index_html

    security.declareProtected(View, 'CookedStyles')
    def CookedStyles(self, REQUEST=None, RESPONSE=None):
        """
        returns the stylesheet
        """
        if RESPONSE: RESPONSE.setHeader('Content-Type', 'text/css')
        return getattr(self, 'css_styles', '')

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self, stx_level=None, setlevel=0):
        """
        return openoffice.org file as html
        """
        return getattr(self, 'cooked_text', 'This OpenOffice.org Document has yet to be uploaded.')

    security.declareProtected(View, 'SearchableBody')
    def SearchableBody(self):
        """
        return html
        """
        return getattr(self, 'cooked_text', '')

    security.declareProtected(View, 'Description')
    def Description(self):
        """
        Dublin core description
        """
        return self.description

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """ Used by the catalog for basic full text indexing """
        return "%s %s %s" % ( self.Title()
                            , self.Description()
                            , self.SearchableBody()
                            )

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, precondition='', file=''):
        """ Update and reindex. """
        self._edit( precondition, file)
        self.reindexObject()

    security.declarePrivate('_edit')
    def _edit(self, precondition='', file=''):
        """ Edit the OODocument - Parses headers and cooks the body """

        if precondition: self.precondition = precondition
        elif self.precondition: del self.precondition

        if self._isNotEmpty(file):
            self.manage_upload(file)

        # cook it
        self._cook()

    security.declarePrivate('_cook')
    def _cook(self):
        """ render the document by xslt to html """

        # get encoding
        encoding=self.getOutEncoding()

        # get a renderer
        renderer=oo_to_html.oo_to_html()

        # render it
        headers, styles, body, images = renderer.cook(self.document(getzip=1), encoding)

        # get images out of the document and create em
        self.workingState(1)
        try:
            for path, size in images.items():
                if len(path) > 4 and path[:4] == "doc/":
                    p=path[4:]
                    try: self.setImage(p, oo_to_html.DynZip(p, caching=0).__of__(self).resize(size))
                    except: pass

        finally: self.workingState(0)

        # filter font-* from stylesheet
        styles = utils.matchfont_regex.sub("", styles)
        
        # store parts into zodb
        self.cooked_text=body
        self.css_styles=styles
        self.setMetadata(headers)


    security.declareProtected(View, 'getImage')
    def getImage(self, path, default=None):
        '''returns the image of the specified zippath.'''
        if not self.hasImage(path): return default
        else: return self._images[path]

    security.declareProtected(View, 'hasImage')
    def hasImage(self, path):
        '''returns true if already there.'''
        if self._images is None: return 0
        try: return self._images.has_key(path)
        except Exception, e:
            return 0

    security.declarePrivate('setImage')
    def setImage(self, path, content):
        """store image if not there."""
        if self._images is None: self._images=OOBTree()
        id=path.split('/')[-1]
        if not self.hasImage(path):
            self._images[path] = OFS.Image.Image(id, id, content)

    security.declarePrivate('workingState')
    def workingState(self, state=None):
        # set temp flag to indicate ongoing document processing
        # to avoid conflicts while image scaling
        if state is None: return getattr(self, '_v_workingState', 0)
        else:
            self._v_workingState=state
            return state

    security.declarePublic('getOutEncoding')
    def getOutEncoding(self, default=None):
        """
        gets and returns encoding from various resources
        """
        encoding=None
        # get output encoding from various resources
        if not encoding: encoding=getattr(getattr(getattr(self, 'portal_properties', None), 'site_properties',None), 'default_charset', None) # plone
        if not encoding: encoding=getattr(self, 'default_zope_data_encoding', None) # icoya
        if not encoding and default: encoding=default
        if not encoding: encoding=sys.getdefaultencoding() # system default

        #if encoding.lower() in ('utf-7', 'utf-8', 'utf-16', 'latin-1'):
            #    # libxml2 does not allow - in certain encoding names
            #    encoding=encoding.replace('-','')

        return encoding

    security.declarePrivate('update_data')
    def update_data(self, data, content_type=None, size=None):
        # store openoffice.org document into zodb
        if content_type is not None: 
            self.content_type=content_type
            self.setFormat(content_type)
        if size is None: size=len(data)
        self._images = OOBTree()
        self.size=size
        self.data=data
        self.ZCacheable_invalidate()
        self.http__refreshEtag()

    def _get_content_type(self, file, body, id, content_type=None):
        # dont use supplied content type inside file
        # since this could return wrong types cause it depends on client
        # i have seen application/x-zip and application/zope-edit
        if type(body) is not type(''): body=body.data
        content_type, enc=guess_content_type(
            getattr(file, 'filename',id), body, content_type)
        return content_type

    security.declareProtected(View, 'document')
    def document(self, getzip=0):
        """
        returns a dynzip instance
        """
        if not getzip:
            return oo_to_html.DynZip(caching=1).__of__(self)
        else:
            return oo_to_html.init_zip(self.data)

    # make zip contents accessable via web
    doc=utils.DynZipWrapper()

    security.declareProtected(ModifyPortalContent, 'setMetadata')
    setMetadata = Document.setMetadata

    security.declareProtected(ModifyPortalContent, 'PUT')
    def PUT(self, REQUEST, RESPONSE):
        """ Handle HTTP PUT requests """
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        type=REQUEST.get_header('content-type', None)
        file=REQUEST['BODYFILE']

        data, size = self._read_data(file)
        content_type=self._get_content_type(file, data, self.__name__,
                                            type or self.content_type)
        self.update_data(data, content_type, size)
        self._cook()
        self.reindexObject()

        RESPONSE.setStatus(204)
        return RESPONSE


def _getViewFor(obj, view='view', default=None):
    ti = obj.getTypeInfo()
    if ti is not None:
        actions = ti.getActions()
        for action in actions:
            if action.get('id', None) == default:
                default=action
            if action.get('id', None) == view:
#                if _verifyActionPermissions(obj, action) and action['action']!='':
#                    return obj.restrictedTraverse(action['action'])
                return obj.restrictedTraverse(action['action'])

        if default is not None:
#            if _verifyActionPermissions(obj, default):
#                return obj.restrictedTraverse(default['action'])
            return obj.restrictedTraverse(default['action'])

        raise 'Unauthorized', ('No accessible views available for %s' %
                               '/'.join(obj.getPhysicalPath()))
    else:
        raise 'Not Found', ('Cannot find default view for "%s"' %
                            '/'.join(obj.getPhysicalPath()))

InitializeClass(OODocument)
