##############################################################################
#
# Copyright (c) 2001-2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""$Id: OOODocument.py,v 1.27 2004/10/09 23:24:55 longsleep Exp $
"""

# html rendered OpenOffice.org/StarOffice Documents 
# for CMF/Plone/Icoya written by Simon Eisenmann

import string
from AccessControl import ClassSecurityInfo
#from Products.CMFCore import CMFCorePermissions
from Products.CMFCore import permissions as CMFCorePermissions
try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No linguaplone support
    from Products.Archetypes.public import *
from MimeTextField import MimeTextField
from Products.Archetypes.BaseUnit import BaseUnit
from Products.Archetypes.ExtensibleMetadata import ExtensibleMetadata
from Products.Archetypes.Marshall import PrimaryFieldMarshaller
from Products.Archetypes.debug import log
from BTrees.OOBTree import OOBTree
from OFS.Image import Image
import oo_to_html
import utils

schema = Schema((

    StringField('id',
                required=1,
                mode="rw",
                accessor="getId",
                mutator="setId",
                default=None,
                widget=IdWidget(label_msgid="label_name",
                                description_msgid="help_name",
                                i18n_domain="plone"),
                ),

    StringField('title',
                required=0,
                searchable=1,
                default='',
                accessor='Title',
                widget=StringWidget(label_msgid="label_title",
                                    description_msgid="help_title",
                                    i18n_domain="plone"),
                ),

    MimeTextField('file',
              required = 0,
              primary = 1,
              accessor = 'CookedBody',
              mutator = '_cook',
              default_method = '_makeDefaultContent',
              allowable_content_types = oo_to_html.openoffice_mime_types,
              output_type='text/html',
              widget = FileWidget(description = "Select the OpenOffice.org file to be uploaded by clicking the 'Browse' button.",
                                  description_msgid = "help_ooofile",
                                  label= "File",
                                  i18n_domain="plone",
                                  label_msgid = "label_ooofile")
             ),
    ), marshall=PrimaryFieldMarshaller()) + ExtensibleMetadata.schema

actions = (
    {
    'id' : 'view',
    'name' : 'View',
    'action' : 'oodocument_view',
    'permissions' : (CMFCorePermissions.View, )
    },
    {
    'id' : 'edit',
    'name' : 'Edit',
    'action' : 'base_edit',
    'permissions' : (CMFCorePermissions.ModifyPortalContent, )
    },
    {
    'id' : 'properties',
    'name' : 'Properties',
    'action' : 'base_metadata',
    'permissions' : (CMFCorePermissions.ModifyPortalContent, )
    },
    )


class OOODocument(BaseContent):
    """OODocuments contain Documents written with
       OpenOffice.org / StarOffice. The content is rendered as html."""

    schema = type = schema
    actions = actions
    
    meta_type = "OOODocument"
    archetype_name = "OOODocument"
    content_icon = "ooo_icon.png"
    
    include_default_actions = 0
    _images = None
    _styles = None
    
    security = ClassSecurityInfo()

    security.declarePrivate('_read') 
    def _read(self, mimetype=None, raw=0, **kwargs):
        """
        helper to map directly to self.Schema()['file']
        """
        file_field = self.Schema()['file']
        return apply(file_field.get, (self, mimetype, raw), kwargs)

    security.declareProtected(CMFCorePermissions.View, 'CookedStyles')
    def CookedStyles(self, REQUEST=None, RESPONSE=None):
        """
        returns the stylesheet
        """
        if RESPONSE: RESPONSE.setHeader('Content-Type', 'text/css')
        return self._styles or ''

    security.declareProtected(CMFCorePermissions.View, 'CookedBody')
    def CookedBody(self, stx_level=None, setlevel=0):
        """
        return openoffice.org file as html
        this is the accessor for the file textfield
        """
	file_field = self.Schema()['file']

        # check if we have content
        if not len(file_field.getEditAccessor(self)()): 
            return "No document has been uploaded yet."
        elif not self.content_type() in oo_to_html.openoffice_mime_types:
            return "Document type %s is not supported." % self.content_type()

        # return our html content
        return self._read()

    security.declareProtected(CMFCorePermissions.View, 'SearchableBody')
    def SearchableBody(self):
        """ return html for indexing """
        file_field = self.Schema()['file']
        if not len(file_field.getEditAccessor(self)()) or not self.content_type() in oo_to_html.openoffice_mime_types:
            return ''
        return self._read()

    security.declareProtected(CMFCorePermissions.View, 'SearchableText')
    def SearchableText(self):
        """ used for catalog fulltext indexing """
        return "%s %s %s" % ( self.Title(),
                              self.Description(),
                              self.SearchableBody()
                            )

    security.declarePublic('content_type')
    def content_type(self):
        # return our content_type
        return self.getContentType('file')

    security.declarePrivate('_cookstyles')
    def _cookstyles(self, styles):
        """ store and sanitize css stylesheets """

        # filter font-* from stylesheet
        styles = utils.matchfont_regex.sub("", styles)

        # store at self
        self._styles = styles

    security.declarePrivate('_cook')                            
    def _cook(self, value, **kwargs):
        """ 
        mutator to do additonal stuff on store
        """
        # store the value        
        file_field = self.Schema()['file']
        apply(file_field.set, (self, value), kwargs)

    # XXX: this is a evil hack to change the id on the fly
    #      when its auto generated
    def post_validate(self, REQUEST, errors):
        try: form = REQUEST.form
        except: return
        new_id = form.get('id', None)
        if not new_id: return
        try: isAutoId = self.isIDAutoGenerated(id=new_id, real=1) # icoya support
        except TypeError: isAutoId = self.isIDAutoGenerated(id=new_id) # try without the real parameter
        if isAutoId:
            # id is auto generated .. use filename if possible
            file = form.get('file_file', None)
            if not file: return
            filename = getattr(file, 'filename', None)
            if not filename: return
            # extract filename from complete path
            filename = filename[max(string.rfind(filename, '/'),
                                    string.rfind(filename, '\\'),
                                    string.rfind(filename, ':'))+1:]
            # change id in request # XXX: evil
            form['id']=filename

    def addSubObjects(self, objects, REQUEST=None):
        """ add in results from transformation """

        # clear image store
        self._images = OOBTree()
        # clear styles
        self._styles = None

        # get components which need more work
        try: headers = objects['headers']
        except: headers = {} 
        try: images = objects['images']
        except: images = {}
        try: styles = objects['styles']
        except: styles = ''

        # store css stylesheets
        self._cookstyles(styles)

        # define known headers (compute mutator)
        known = ('title', 'description', 'language')
        for h in known:
            if headers.has_key(h):
                # get mutator
                mutator = getattr(self, 'set%s' % (h.capitalize()))
                # store headers by calling the mutators
                mutator(headers.get(h))

        # define known headers (define mutator)
        known = ( ('keyword', 'setSubject'), )
        for h,m in known:
            if headers.has_key(h):
                # get mutator
                mutator = getattr(self, m)
                # store headers by calling the mutators
                mutator(headers.get(h))

        # extract, resize, store included images
        for path, size in images.items():
            if len(path) > 4 and path[:4] == "doc/":
                p=path[4:]
                try: self.setImage(p, oo_to_html.DynZip(p, caching=0).__of__(self).resize(size))
                except: pass 

    security.declareProtected(CMFCorePermissions.View, 'document')                
    def document(self, getzip=0):
        """
        returns a dynzip instance
        """
        if not getzip:
            return oo_to_html.DynZip(caching=1).__of__(self)
        else:
            return oo_to_html.init_zip(self._read(raw=1))

    # make zip contents accessable via web
    doc=utils.DynZipWrapper()          
   
    security.declarePrivate('_makeDefaultContent') 
    def _makeDefaultContent(self):
        """
        creates an empty baseunit containing an empty oo writer document
        as default_method for file field
        """
        file_field = self.Schema()['file']
        bu = BaseUnit(file_field.getName(),
                      oo_to_html.loadEmptyOODocument(),
                      mimetype=oo_to_html.openoffice_mime_types[0],
                      instance=self)
        return bu

    security.declareProtected(CMFCorePermissions.View, 'getImage')    
    def getImage(self, path, default=None):
        """
        returns the image of the specified zippath
        """
        if not self.hasImage(path): return default
        else: return self._images[path]

    security.declareProtected(CMFCorePermissions.View, 'hasImage')        
    def hasImage(self, path):
        """
        returns true if already there
        """
        if self._images is None: return 0
        try: return self._images.has_key(path)
        except Exception, e:
            return 0
    
    security.declarePrivate('setImage')
    def setImage(self, path, content):
        """
        store image if not there
        """
        if self._images is None: self._images=OOBTree()
        id=path.split('/')[-1]
        if not self.hasImage(path):
            self._images[path] = Image(id, id, content)

    security.declarePrivate('workingState')
    def workingState(self, state=None):
        # set temp flag to indicate ongoing document processing
        # to avoid conflicts while image scaling
        if state is None: return getattr(self, '_v_workingState', 0)
        else:
            self._v_workingState=state
            return state
            
registerType(OOODocument)
