#!/usr/bin/python2
#
# simple example how to use python to convert an openoffice.org document
# to html with an xsl transformation
#
# (c) Simon Eisenmann, sturktur AG <simon@struktur.de>

import libxml2
import libxslt
import zipfile
import tempfile
import os

# data locations
source="data/example.sxw"
destination="out/example.html"

# register helper method for xslt
def f(ctx): 
    # dummy method
    return "0" 
libxslt.registerExtModuleFunction("current-time-millis", "http://www.jclark.com/xt/java/java.lang.System", f)

# deactivate error messages from the validation
def noerr(ctx, str):
    pass
libxml2.registerErrorHandler(noerr, None)
libxslt.registerErrorHandler(noerr, None)

# open files
fp=open(source, 'rb', -1)
zip=zipfile.ZipFile(fp, 'rb')

# write temporary files 
tmpfiles=[]
params={}
for f in (('stylesFileURL','styles.xml'), ('metaFileURL','meta.xml')):
	t=tempfile.mktemp()
	tmpfiles.append(t)
	file=open(t, 'wb', -1)
	file.write(zip.read(f[1]))
	file.close()
	params.update({f[0]: "'%s'" % (t)}) 

# xslt parameters
params.update({ "absoluteSourceDirRef": "'%s'" % (os.getcwd()),
		"disableJava": "true()",
		"disableLinkedTableOfContent": "true()",
        	"DEBUG": "true()"
       	      })

# read stylesheet
styledoc=libxml2.parseFile("../sx2ml/main_html.xsl")
# parse stylesheet
style=libxslt.parseStylesheetDoc(styledoc)
# read ooo file and parse it
doc=libxml2.parseDoc(zip.read('content.xml'))
# apply stylesheet
result=style.applyStylesheet(doc, params)
# store to file
style.saveResultToFilename(destination, result, 0)
# cleanups
style.freeStylesheet()
doc.freeDoc()
result.freeDoc()

# close
zip.close()
fp.close()

# remove tempfiles
for t in tmpfiles:
    os.unlink(t)
	
	
